@@ -22,6 +22,11 @@ class APIerror(Exception):
         return repr(self)
 
 
+class InternalError(APIerror):
+    def __init__(self, message, status_code=None, payload=None):
+        super().__init__(message, status_code=status_code, payload=payload)
+
+
 class BadRequest(APIerror):
     def __init__(self, message, status_code=None, payload=None):
         super().__init__(message, status_code=status_code, payload=payload)
@@ -37,7 +42,8 @@ class RequestNotUnderstood(BadRequest):
 
 
 class RequestNotAuthorized(BadRequest):
-    def __init__(self, message):
+    def __init__(self, message, debug_message=''):
+        self.debug_message = debug_message
         super().__init__(message, status_code=403)
 
 
@@ -58,3 +64,7 @@ class ProblemDecodingStoredQueryOut(Exception):
     """
     problem with storage? race?
     """
+
+
+class MissingRequestParameter(BadRequest):
+    pass
\ No newline at end of file
