@@ -20,26 +20,26 @@ Module API
 
 from __future__ import absolute_import, division, print_function
 
+import itertools
+import os
 from builtins import (bytes, str, open, super, range,
                       zip, round, input, int, pow, object, map, zip)
 
 import string
 import json
 import logging
-import re
 import yaml
 
-import os
-import  numpy as np
+import numpy as np
 from astropy.table import Table
 
 from cdci_data_analysis.analysis.queries import _check_is_base_query
-from ..analysis import tokenHelper
+from ..analysis import tokenHelper, parameters
 from .catalog import BasicCatalog
 from .products import QueryOutput
 from .queries import ProductQuery, SourceQuery, InstrumentQuery
-from .io_helper import FilePath
-from .exceptions import RequestNotUnderstood, RequestNotAuthorized
+from .io_helper import upload_file
+from .exceptions import RequestNotUnderstood, RequestNotAuthorized, InternalError
 
 __author__ = "Andrea Tramacere"
 
@@ -54,9 +54,16 @@ __author__ = "Andrea Tramacere"
 # Project
 # relative import eg: from .mod import f
 
+# list of parameters not to be included in the par_dic object
+
+# TODO: this is not preserved between requests, and is not thread safe. Why not pass it in class instances?
+params_not_to_be_included = ['user_catalog',]
+
+
 class DataServerQueryClassNotSet(Exception):
     pass
 
+
 class Instrument:
     def __init__(self,
                  instr_name,
@@ -115,11 +122,48 @@ class Instrument:
     def _check_names(self):
         pass
 
+    def set_pars_from_dic(self, par_dic, verbose=False):
+        product_type = par_dic.get('product_type', None)
+        if product_type is not None:
+            query_name = self.get_product_query_name(product_type)
+            query_obj = self.get_query_by_name(query_name)
+            # loop over the list of parameters for the requested query,
+            # but also of the instrument query and source query
+            param_list = (query_obj.parameters +
+                          self.instrumet_query.parameters +
+                          self.src_query.parameters)
+        else:
+            param_list = [par for _query in self._queries_list for par in _query.parameters]
 
-    def set_pars_from_dic(self,par_dic,verbose=False):
-        for _query in self._queries_list:
-            for par in _query._parameters_list:
-                par.set_from_form(par_dic,verbose=verbose)
+        updated_par_dic = par_dic.copy()
+        
+        for par in param_list:
+            self.logger.info("before normalizing, set_pars_from_dic>> par: %s par.name: %s par.value: %s par_dic[par.name]: %s",
+                             par, par.name, par.value, par_dic.get(par.name, None))
+
+            # this is required because in some cases a parameter is set without a name (eg UserCatalog),
+            # or they don't have to set (eg scw_list)
+            if par.name is not None and par.name not in params_not_to_be_included:
+                # set the value for par to a default format,
+                # or to a default value if this is not included within the request
+                updated_par_dic[par.name] = par.set_value_from_form(par_dic, verbose=verbose)
+
+                if par.units_name is not None:
+                    if par.default_units is not None:
+                        updated_par_dic[par.units_name] = par.default_units
+                    else:
+                        raise InternalError("Error when setting the parameter %s: "
+                                            "default unit not specified" % par.name)
+                else:
+                    self.logger.warning("units_name for the parameter %s not specified", par.name)
+
+            self.logger.info("after normalizing, set_pars_from_dic>> par: %s par.name: %s par.value: %s par_dic[par.name]: %s",
+                             par, par.name, par.value, par_dic.get(par.name, None))
+
+            if par.name == "scw_list":
+                self.logger.info("set_pars_from_dic>> scw_list is %s", par.value)
+
+        return updated_par_dic
 
     def set_par(self,par_name,value):
         p=self.get_par_by_name(par_name)
@@ -136,7 +180,7 @@ class Instrument:
 
         return p
 
-    def test_communication(self,config,logger=None):
+    def test_communication(self, config, logger=None):
         if self.data_server_query_class is not None:
             return self.data_server_query_class(config=config, instrument=self).test_communication(logger=logger)
         else:
@@ -150,6 +194,50 @@ class Instrument:
         if self.data_server_query_class is not None:
             return self.data_server_query_class(config=config,instrument=self).test_has_input_products(instrument,logger=logger)
 
+    def parse_inputs_files(self,
+                           par_dic,
+                           request,
+                           temp_dir,
+                           verbose,
+                           use_scws,
+                           sentry_client=None):
+        error_message = 'Error while {step} from the frontend{temp_dir_content_msg}'
+        # TODO probably exception handling can be further improved and/or optmized
+        try:
+            # set catalog
+            step = 'uploading catalog file'
+            self.upload_catalog_from_fronted(par_dic=par_dic, request=request, temp_dir=temp_dir)
+            step = 'setting catalog file'
+            self.set_catalog(par_dic)
+
+            # set scw_list
+            step = 'uploading scw_list file'
+            input_file_path = self.upload_input_products_from_fronted(name='user_scw_list_file',
+                                                                      request=request,
+                                                                      temp_dir=temp_dir)
+            step = 'setting input scw_list file'
+            self.set_input_products_from_fronted(input_file_path=input_file_path, par_dic=par_dic, verbose=verbose)
+        except Exception as e:
+            error_message = error_message.format(step=step,
+                                                 temp_dir_content_msg='' if not os.path.exists(temp_dir) else
+                                                 f', content of the temporary directory is {os.listdir(temp_dir)}')
+
+            if sentry_client is not None:
+                sentry_client.capture('raven.events.Message',
+                                      message=f'{error_message}\n{e}')
+            raise RequestNotUnderstood(error_message)
+
+        if input_file_path is None and use_scws == 'user_file':
+            error_message = 'scw_list file was expected to be passed, ' \
+                            'but it has not been found, please check the inputs'
+
+            raise RequestNotUnderstood(error_message)
+        elif input_file_path is not None and use_scws != 'user_file':
+            error_message = 'scw_list file was found despite ' \
+                            'use_scws was indicating this was not provided, please check the inputs'
+
+            raise RequestNotUnderstood(error_message)
+
     def run_query(self, product_type,
                   par_dic,
                   request,
@@ -175,27 +263,9 @@ class Instrument:
 
         # set pars values from the input parameters
         query_out = self.set_pars_from_form(par_dic, verbose=verbose, sentry_client=sentry_client)
-
         if verbose:
             self.show_parameters_list()
 
-        # set catalog
-        if query_out.status_dictionary['status']==0:
-            query_out=self.set_catalog_from_fronted(par_dic, request,back_end_query, logger=logger, verbose=verbose,sentry_client=sentry_client)
-
-        # set input products
-        if query_out.status_dictionary['status'] == 0:
-            try:
-                query_out = self.set_input_products_from_fronted(par_dic, request,back_end_query, logger=logger,verbose=verbose,sentry_client=sentry_client)
-            except Exception as e:
-                # FAILED
-                query_out.set_failed(product_type,
-                                     message='wrong parameter',
-                                     logger=logger,
-                                     sentry_client=sentry_client,
-                                     excep=e)
-
-        # self.logger.info('--> par dict', par_dic)
         logger.info('--> par dict', par_dic)
 
         if dry_run:
@@ -212,12 +282,13 @@ class Instrument:
                     query_name = self.get_product_query_name(product_type)
                     query_obj = self.get_query_by_name(query_name)
                     roles = []
+                    
                     if decoded_token is not None: # otherwise the request is public
                         roles = tokenHelper.get_token_roles(decoded_token)
+
                     # assess the permissions for the query execution
                     self.check_instrument_query_role(query_obj, product_type, roles, par_dic)
 
-                    query_obj = self.get_query_by_name(query_name)
                     query_out = query_obj.run_query(self, out_dir, job, run_asynch,
                                                     query_type=query_type,
                                                     config=config,
@@ -245,15 +316,40 @@ class Instrument:
                 except RequestNotUnderstood as e:
                     logger.warning("bad request from user, passing through: %s", e)
                     raise
+                except InternalError as e:
+                    if hasattr(e, 'message') and e.message is not None:
+                        message = e.message
+                    else:
+                        message = ('Your request produced an unusual result. It might not be what you expected. '
+                                     'It is possible that this particular parameter selection should indeed lead to this outcome '
+                                     '(e.g. there is no usable data). Please look carefully on your request.\n\n'
+                                     'It is also possible that the platform experienced a temporary issue. '
+                                     'We aim at distinguishing all of such issues and report them clearly, '
+                                     'but for now, we unfortunately can not be certain all cases like this are detected. '
+                                     'We try to discover on our own and directly address any temporary issue. '
+                                     'But some issues might slip past us. If you are willing to help us, '
+                                     'please use "provide feedback" button below. We would greatly appreciate it!\n\n'
+                                     'This additional information might help:\n\n'
+                               )
+                    e_message = f'Instrument: {self.name}, product: {product_type} failed!\n'
+                    query_out.set_failed(product_type,
+                                         message=message,
+                                         e_message=e_message,
+                                         logger=logger,
+                                         sentry_client=sentry_client,
+                                         excep=e)
+
                 except Exception as e: # we shall not do that
                     logger.error("run_query failed: %s", e)
                     # logger.error("run_query failed: %s", traceback.format_exc())
                     query_out.set_failed(product_type, logger=logger, sentry_client=sentry_client, excep=e)
 
         # adding query parameters to final products
+        # TODO: this can be misleading since it's the parameters actually used
         query_out.set_analysis_parameters(par_dic)
-        query_out.set_api_code(par_dic)
-        query_out.dump_analysis_parameters(out_dir,par_dic)
+        # TODO perhaps this will change
+        query_out.set_api_code(par_dic, url=back_end_query.config.products_url + "/dispatch-data")
+        query_out.dump_analysis_parameters(out_dir, par_dic)
 
         return query_out
 
@@ -266,23 +362,43 @@ class Instrument:
     def check_instrument_query_role(self, query_obj, product_type, roles, par_dic):
         results = query_obj.check_query_roles(roles, par_dic)
         if not results['authorization']:
-            raise RequestNotAuthorized(f"Roles {roles} not authorized to request the product {product_type}, {results['needed_roles']} roles are needed")
+            results['needed_roles']
+
+            lacking_roles = sorted(list(set(results['needed_roles']) - set(roles)))
+
+            lacking_roles_comment = "\n".join([
+                f" - {role}: {results.get('needed_roles_with_comments', {}).get(role, 'please refer to support for details')}"
+                for role in lacking_roles
+            ])
+
+            raise RequestNotAuthorized(
+                f"Unfortunately, your priviledges are not sufficient to make the request for this particular product and parameter combination.\n"
+                f"- Your priviledge roles include {roles}\n"
+                f"- You are lacking all of the following roles:\n"
+                f"{lacking_roles_comment}\n"
+                f"You can request support if you think you should be able to make this request."
+            )
         else:
             return True
 
     def get_html_draw(self, prod_name, image,image_header,catalog=None,**kwargs):
         return self.get_query_by_name(prod_name).get_html_draw( image,image_header,catalog=catalog,**kwargs)
 
+    #def get_par_by_name(self,par_name, validate=False):
     def get_par_by_name(self,par_name):
         p=None
 
         for _query in self._queries_list:
             if par_name in _query.par_names:
+                # TODO: this picks the last one if there are many?..
                 p  =  _query.get_par_by_name(par_name)
 
         if p is None:
             raise Warning('parameter', par_name, 'not found')
 
+     #   if validate and hasattr(p, 'check_value'):
+     #       p.check_value(p.value)
+
         return p
 
     def show_parameters_list(self):
@@ -351,7 +467,6 @@ class Instrument:
 
         return l
 
-
     def set_pars_from_form(self,par_dic,logger=None,verbose=False,sentry_client=None):
         #print('---------------------------------------------')
         #print('setting form paramters')
@@ -366,9 +481,14 @@ class Instrument:
             self.set_pars_from_dic(par_dic,verbose=verbose)
             #DONE
             q.set_done(debug_message=str(debug_message))
+        except RequestNotUnderstood as e:
+           q.set_failed(f"please adjust request parameters: {e.message}",
+                        logger=logger,
+                        sentry_client=None,
+                        excep=e)
+
         except Exception as e:
             #FAILED
-
             m = f'problem setting form parameters from dict: {par_dic}'
             logger.error(m)
 
@@ -376,87 +496,30 @@ class Instrument:
                          logger=logger,
                          sentry_client=sentry_client,
                          excep=e)
-
             #status=1
             #error_message= 'error in form parameter'
             #debug_message = e
             #logger.exception(e)
-
-
         #print('---------------------------------------------')
         return q
 
-
-    def set_input_products_from_fronted(self,par_dic,request,back_end_query,verbose=False,logger=None,sentry_client=None):
-        #print('---------------------------------------------')
-        #print('setting user input prods')
-        input_prod_list_name = self.instrumet_query.input_prod_list_name
-        q = QueryOutput()
-        #status = 0
-        error_message = ''
-        debug_message = ''
-        input_file_path=None
-
-        if logger is None:
-            logger = logging.getLogger(repr(self))
-
+    def upload_input_products_from_fronted(self, name, request, temp_dir):
+        input_file_path = None
         if request.method == 'POST':
-            try:
-                input_file_path = back_end_query.upload_file('user_scw_list_file', back_end_query.scratch_dir)
-                #DONE
-                q.set_done( debug_message=str(debug_message))
-            except Exception as e:
-                #DONE
-                q.set_failed('failed to upload scw_list file',
-                             extra_message='failed to upload %s' % self.input_prod_name,
-                             sentry_client=sentry_client,
-                             excep=e)
-
-
-            try:
-                has_input=self.set_input_products(par_dic,input_file_path,input_prod_list_name)
-                #DONE
-                q.set_done( debug_message=str(debug_message))
-            except Exception as e :
-                #FAILED
-                q.set_failed('scw_list file is not valid',
-                             extra_message='scw_list file is not valid, please check the format',
-                             logger=logger,
-                             sentry_client=sentry_client,
-                             excep=e)
-
-
-
-            #print ('has input',has_input)
-            try:
-
-                if has_input==True:
-                    pass
-                else:
-                    raise RuntimeError
-                #DONE
-                q.set_done( debug_message=str(debug_message))
-
-            except Exception as e:
-                #FAILED
-                q.set_failed('setting input scw_list',
-                             extra_message='scw_list file is not valid, please check the format',
-                             sentry_client=sentry_client,
-                             excep=e)
-
-
-        self.set_pars_from_dic(par_dic,verbose=verbose)
-
-        #print('---------------------------------------------')
-        return q
-
-
-
-
+            # save to a temporary folder, and delete it afterwards
+            input_file_path = upload_file(name, temp_dir)
+        return input_file_path
 
+    def set_input_products_from_fronted(self, input_file_path, par_dic, verbose=False):
+        input_prod_list_name = self.instrumet_query.input_prod_list_name
+        if input_file_path is not None:
+            has_input = self.set_input_products(par_dic, input_file_path, input_prod_list_name)
+            if has_input:
+                pass
+            else:
+                raise RuntimeError
 
     def set_input_products(self, par_dic, input_file_path,input_prod_list_name):
-        has_prods=False
         if input_file_path is None:
             #if no file we pass OK condition
             #since the paramter will be passed from the form
@@ -481,120 +544,44 @@ class Instrument:
 
         return has_prods
 
-        # template = re.compile(r'^(\d{12}).(\d{3})$')
-        # if input_file_path is None:
-        #     return True
-        # else:
-        #     with open(input_file_path) as f:
-        #         _lines = f.readlines()
-        #
-        #     # should now accept any combination of commas and/or newlines
-        #     # raise error if any of the scwlist is not matching the template
-        #     lines = []
-        #     for ll in _lines:
-        #         lines.extend(ll.split(","))
-        #
-        #
-        #     acceptList = [item.strip() for item in lines if template.match(item)]
-        #
-        #
-        #
-        #
-        #     if len(acceptList)!=len(lines):
-        #         raise RuntimeError
-        #
-        #     par_dic[input_prod_list_name]=acceptList
-        #     #print ("--> accepted scws",acceptList,len(acceptList))
-        #     return len(acceptList)>=1
-
-
-    def set_catalog_from_fronted(self,par_dic,request,back_end_query,logger=None,verbose=False,sentry_client=None):
-        #print('---------------------------------------------')
-        #print('setting user catalog')
-
-        q = QueryOutput()
-        #status = 0
-        error_message = ''
-        debug_message = ''
-
-        if logger is None:
-            logger = logging.getLogger(__name__)
-
-        cat_file_path=None
+    def upload_catalog_from_fronted(self, par_dic, request, temp_dir):
         if request.method == 'POST':
-            #print('POST')
-            try:
-                cat_file_path = back_end_query.upload_file('user_catalog_file', back_end_query.scratch_dir)
+            # save to a temporary folder, and delete it afterwards
+            cat_file_path = upload_file('user_catalog_file', temp_dir)
+            if cat_file_path is not None:
                 par_dic['user_catalog_file'] = cat_file_path
-                #print('set_catalog_from_fronted,request.method', request.method, par_dic['user_catalog_file'],cat_file_path)
-                #DONE
-                q.set_done( debug_message=str(debug_message))
-            except Exception as e:
-                #FAILED
-                q.set_failed('upload catalog file',
-                             extra_message='failed to upload catalog file',
-                             logger=logger,
-                             sentry_client=sentry_client,
-                             excep=e)
 
-
-
-        try:
-            self.set_catalog(par_dic, scratch_dir=back_end_query.scratch_dir)
-            #DONE
-            q.set_done(debug_message=str(debug_message))
-        except Exception as e:
-            # FAILED
-            q.set_failed('set catalog file',
-                         extra_message='failed to set catalog',
-                         logger=logger,
-                         sentry_client=sentry_client,
-                         excep=e)
-
-
-
-        self.set_pars_from_dic(par_dic,verbose=verbose)
-
-        #print('setting user catalog done')
-        #print('---------------------------------------------')
-        return q
-
-    def set_catalog(self, par_dic, scratch_dir='./'):
-
-        user_catalog_file=None
-        if 'user_catalog_file' in par_dic.keys() :
+    def set_catalog(self, par_dic):
+        user_catalog_file = None
+        if 'user_catalog_file' in par_dic.keys():
             user_catalog_file = par_dic['user_catalog_file']
-            #print("--> user_catalog_file ",user_catalog_file)
 
-        if 'user_catalog_dictionary'in par_dic.keys() and par_dic['user_catalog_dictionary'] is not None:
-            if type(par_dic['user_catalog_dictionary'])==dict:
-                self.set_par('user_catalog',build_catalog(par_dic['user_catalog_dictionary']))
+        if 'user_catalog_dictionary' in par_dic.keys() and par_dic['user_catalog_dictionary'] is not None:
+            if type(par_dic['user_catalog_dictionary']) == dict:
+                self.set_par('user_catalog', build_catalog(par_dic['user_catalog_dictionary']))
             else:
                 catalog_dic = json.loads(par_dic['selected_catalog'])
                 self.set_par('user_catalog', build_catalog(catalog_dic))
-
+        # setting user_catalog in the par_dic, either loading it from the file or aas an object
         if user_catalog_file is not None:
-            # print('loading catalog  using file', user_catalog_file)
-            self.set_par('user_catalog', load_user_catalog(user_catalog_file))
-            # print('user catalog done, using file',user_catalog_file)
-
+            catalog_object = load_user_catalog(user_catalog_file)
+            self.set_par('user_catalog', catalog_object)
+            # TODO set selected_catalog ? in this way it will show up in the email url link
+            # and used consistently for the job_id generation
+            self.set_par('selected_catalog', json.dumps(catalog_object.get_dictionary()))
+            # TODO verify that the file path is not needed in the frontend (should not)
+            par_dic.pop('user_catalog_file', None)
         else:
             if 'catalog_selected_objects' in par_dic.keys():
-
                 catalog_selected_objects = np.array(par_dic['catalog_selected_objects'].split(','), dtype=np.int)
             else:
                 catalog_selected_objects = None
-
             if 'selected_catalog' in par_dic.keys():
-                catalog_dic=json.loads(par_dic['selected_catalog'])
-
+                catalog_dic = json.loads(par_dic['selected_catalog'])
                 user_catalog = build_catalog(catalog_dic, catalog_selected_objects)
                 self.set_par('user_catalog', user_catalog)
 
 
-
-
-
 def load_user_catalog(user_catalog_file):
     return BasicCatalog.from_file(user_catalog_file)
 
