@@ -1,7 +1,10 @@
 from __future__ import absolute_import, division, print_function
 
+import os
 from builtins import (bytes, str, open, super, range,
                       zip, round, input, int, pow, object, map, zip)
+import shutil
+import tempfile
 
 __author__ = "Andrea Tramacere"
 
@@ -19,30 +22,15 @@ __author__ = "Andrea Tramacere"
 import json
 from collections import OrderedDict
 
-#from pathlib import Path
-
-#from astropy import wcs
-#from astropy.wcs import WCS
-
-from astropy.io import fits as pf
-
 import matplotlib
-import copy
 
-matplotlib.use('Agg') #, warn=False - deprecated
+matplotlib.use('Agg')  #, warn=False - deprecated
 
-#import matplotlib.pyplot as plt
+from .plot_tools import Image, ScatterPlot, GridPlot
 
-#import numpy as np
+from oda_api.data_products import NumpyDataProduct, NumpyDataUnit
 
-#import mpld3
-#from mpld3 import plugins
-
-from .plot_tools import  Image,ScatterPlot,GridPlot
-
-from  oda_api.data_products import NumpyDataProduct, NumpyDataUnit
-
-from  oda_api.api import  DispatcherAPI
+from oda_api.api import DispatcherAPI
 from .parameters import *
 from .io_helper import FilePath
 from .io_helper import view_traceback, FitsFile
@@ -51,7 +39,9 @@ from .json import CustomJSONEncoder
 from .exceptions import ProblemDecodingStoredQueryOut
 
 import traceback
-import logging
+
+from typing import Union
+
 from ..app_logging import app_logging
 
 try:
@@ -60,9 +50,7 @@ except ImportError:
     from urllib import urlencode
 
 
-
-
-
+logger = app_logging.getLogger(__name__)
 
 
 class QueryOutput(object):
@@ -75,28 +63,43 @@ class QueryOutput(object):
 
         self.set_status(0, job_status='unknown')
 
-
     def set_instrument_parameters(self,o):
-        self.prod_dictionary['instrumet_parameters'] = o
+        self.prod_dictionary['instrument_parameters'] = o
 
     def set_analysis_parameters(self, query_dict):
         self.prod_dictionary['analysis_parameters'] = query_dict
 
-    def set_api_code(self,query_dict):
-        self.prod_dictionary['api_code'] = DispatcherAPI.set_api_code(query_dict)
+    def set_api_code(self, query_dict, url=None):
+        self.prod_dictionary['api_code'] = DispatcherAPI.set_api_code(query_dict, url=url)
 
-    def dump_analysis_parameters(self,work_dir,query_dict):
-        file_path=FilePath(file_dir=work_dir, file_name='analysis_parameters.json')
-        with open(file_path.path, 'w')  as outfile:
-            my_json_str = json.dumps(query_dict, indent=4)
-            outfile.write(u'%s' % my_json_str)
+    def dump_analysis_parameters(self, work_dir, query_dict):
+        file_path = FilePath(file_dir=work_dir, file_name='analysis_parameters.json')
+        if not os.path.exists(file_path.path):
+            with open(file_path.path, 'w')  as outfile:
+                _dict = {k: v for k, v in query_dict.items() if v is not None}
+                my_json_str = json.dumps(_dict, indent=4, sort_keys=True)
+                # TODO further test if formatting is really needed
+                outfile.write(u'%s' % my_json_str)
+        else:
+            with open(file_path.path) as outfile:
+                dict_analysis_parameters = json.load(outfile)
+            logger.info("analysis_parameters.json file already present, "
+                        "and originally used for the job_id calculation: %s"
+                        " while newly requested parameters are %s",
+                        json.dumps(dict_analysis_parameters, sort_keys=True, indent=4),
+                        json.dumps({k: v for k, v in query_dict.items() if v is not None}, sort_keys=True, indent=4))
 
     def set_products(self, keys, values):
         for k, v in zip(keys, values):
             self.prod_dictionary[k] = v
 
     def set_done(self, message='', debug_message='', job_status=None, status=0,comment='',warning=''):
-        self.set_status(status, message=message, debug_message=debug_message, job_status=job_status,comment=comment,warning=warning)
+        self.set_status(status,
+                        message=message,
+                        debug_message=debug_message,
+                        job_status=job_status,
+                        comment=comment,
+                        warning=warning)
 
     def set_failed(self, failed_operation,
                    message_prepend_str='',
@@ -130,11 +133,9 @@ class QueryOutput(object):
     def _set_job_status(self, job_status):
         if job_status is not None:
             if job_status in self._allowed_job_status_values_:
-
                 self.status_dictionary['job_status'] = job_status
             else:
-                raise RuntimeError('job_status', job_status, ' in QueryOutput is not allowed',
-                                   self._allowed_job_status_values_)
+                raise RuntimeError(f'job_status {job_status} in QueryOutput is not allowed, allowed {self._allowed_job_status_values_}')
 
     def set_status(self, status, message='', error_message='', debug_message='', job_status=None,comment='',warning=''):
 
@@ -161,9 +162,10 @@ class QueryOutput(object):
     def get_job_status(self):
         return self.status_dictionary['job_status']
 
-    def set_query_exception(self, excep,
+    def set_query_exception(self,
+                            excep: Union[Exception, None],  # used as `error_message`, as repr, only if `e_message` is None
                             failed_operation,
-                            message_prepend_str='',
+                            message_prepend_str='',  # used in `message`
                             extra_message=None,
                             message=None,
                             logger_prepend_str='==>',
@@ -188,7 +190,6 @@ class QueryOutput(object):
                     try:
                         e_message = excep.__repr__()
                     except:
-
                         e_message = ''
         else:
             print('e_message', e_message)
@@ -231,7 +232,7 @@ class QueryOutput(object):
         return f'[ {self.__class__.__name__} ]'
 
     def serialize(self, writable):
-        json.dump({ 
+        json.dump({
                     k: v
                     for k, v in self.__dict__.items()
                     if not k.startswith("_")
@@ -239,7 +240,7 @@ class QueryOutput(object):
                   writable,
                   cls=CustomJSONEncoder
                 )
-    
+
     def deserialize(self, readable):
 
         logger = app_logging.getLogger(self.__class__.__name__)
@@ -275,11 +276,6 @@ class QueryProductList(object):
         return prod
 
 
-
-
-
-
-
 class BaseQueryProduct(object):
     # TODO move this to NumpyDataProd Class
     def __init__(self,
@@ -291,25 +287,17 @@ class BaseQueryProduct(object):
                  meta_data={}):
 
         self.name = name
-        self.meta_data=meta_data
+        self.meta_data = meta_data
 
         if file_name is not None:
-            #print('set file phat')
-            #print('workig dir', file_dir)
-            #print('file name', file_name)
-            #print('name_prefix', name_prefix)
             self.file_path = FilePath(file_name=file_name, file_dir=file_dir, name_prefix=name_prefix)
-            #print('file_path set to', self.file_path.path)
 
         if data is not None:
-
-            if isinstance(data,NumpyDataProduct):
+            if isinstance(data, NumpyDataProduct):
                 self.data = data
             else:
                 raise RuntimeError('data is not of the expected type',type(NumpyDataProduct))
 
-
-
     def encode(self):
         return self.data.encode()
 
@@ -325,35 +313,42 @@ class BaseQueryProduct(object):
         else:
             file_path = self.file_path.get_file_path(file_name=file_name, file_dir=file_dir)
 
+        # turns out the way astropy writes some structures really overloads some nfs mounts.
+        # this happened on UNIGE site after update. 
+        # workaround is made here and should be configurable
+        logger.debug("\033[31mbefore %s.write(file_name=%s) as %s\033[0m", self, file_name, file_path)
 
-        self.data.write_fits_file(file_path, overwrite=overwrite)
+        with tempfile.NamedTemporaryFile(delete=False) as f:
+            self.data.write_fits_file(f, overwrite=overwrite)
+            f.close()
+            shutil.move(f.name, file_path)
+        
+#        self.data.write_fits_file(file_path, overwrite=overwrite)
+        logger.debug("\033[31mafter %s.write(file_name=%s) as %s\033[0m", self, file_name, file_path)
 
-    def add_url_to_fits_file(self,par_dict,url='',use_primary=True,add_query_dict=True):
+    def add_url_to_fits_file(self, 
+                             par_dict, 
+                             url='', 
+                             use_primary=True, 
+                             add_query_dict=True):
         url = '%s?%s' % (url, urlencode(par_dict))
 
         url_dict = OrderedDict()
-        url_dict['url']=url
-        #self.data.show()
+        url_dict['url'] = url
+        
         if use_primary is True:
-            du=self.data.get_data_unit_by_name('PRIMARY')
-            #print('du PRIMARY',du)
+            du = self.data.get_data_unit_by_name('PRIMARY')
             if du is None:
                 du = self.data.get_data_unit_by_name('Primary')
-                #print('du PRIMARY', du)
             if du is None:
-                du=self.data.get_data_unit(0)
-                #print('du.hdu_type', du.hdu_type)
+                du = self.data.get_data_unit(0)
             if du is not None:
-                #print('du.hdu_type',du.hdu_type)
-                if du.hdu_type!='primary':
-                    du=None
+                if du.hdu_type != 'primary':
+                    du = None
 
         if du is None or use_primary is False:
-            #print('du not found' )
-            du=NumpyDataUnit(None, name='Primary', hdu_type='primary')
-            self.data.data_unit.insert(0,du)
-
-
+            du = NumpyDataUnit(None, name='Primary', hdu_type='primary')
+            self.data.data_unit.insert(0, du)
 
         if add_query_dict is True:
             url_dict.update(par_dict)
@@ -370,17 +365,21 @@ class BaseQueryProduct(object):
 
         _url_dict.update(du.header)
 
-        du.header=_url_dict
+        du.header = _url_dict
         #du.header.append(url_dict.keys(), end=True)
         #for k in url_dict.keys():
         #    du.header[k]=url_dict[k]
         #self.data.show()
 
 class ImageProduct(BaseQueryProduct):
-    def __init__(self,name='', data=None, file_name='image.fits',meta_data={}, **kwargs):
+    def __init__(self, name='', data=None, file_name='image.fits',meta_data={}, **kwargs):
 
         self.file_name = file_name
-        super(ImageProduct, self).__init__(name=name,data=data, file_name=file_name, meta_data=meta_data,**kwargs)
+        super(ImageProduct, self).__init__(name=name,
+                                           data=data,
+                                           file_name=file_name,
+                                           meta_data=meta_data,
+                                           **kwargs)
 
 
     def get_html_draw(self, catalog=None, data_ID=0):
@@ -410,23 +409,24 @@ class LightCurveProduct(BaseQueryProduct):
                  meta_data={},
                  **kwargs):
 
-
-
-
         self.name = name
         self.data = data
         self.file_name = file_name
-        #self.src_name = src_name
 
-        super(LightCurveProduct, self).__init__(name=name, data=data,file_name=file_name,meta_data=meta_data, **kwargs)
+        super(LightCurveProduct, self).__init__(name=name,
+                                                data=data,
+                                                file_name=file_name,
+                                                meta_data=meta_data,
+                                                **kwargs)
 
-
-
-    def get_html_draw(self,x,y,dy=None,dx=None,x_label='',y_label='',title=None,max_bins=1E4):
-        warning=''
-        max_bins=np.int(max_bins)
-        if (np.size(x)>max_bins):
-            #print(np.size(x),np.shape(x),max_bins,x[:])
+    def get_html_draw(self,
+                      x, y,
+                      dy=None, dx=None,
+                      x_label='', y_label='',
+                      title=None, max_bins=1E4):
+        warning = ''
+        max_bins = np.int(max_bins)
+        if np.size(x) > max_bins:
             actual_size = np.size(x)
             x=x[:max_bins]
             y=y[:max_bins]
@@ -629,11 +629,18 @@ class SpectralFitProduct(BaseQueryProduct):
 
         if rmf_file is not None and rmf_file!='None':
             self.rmf_file = FilePath(file_name=rmf_file, file_dir=file_dir).path
+        else:
+            logger.warning("\033[31mrmf_file %s is not good!\033[0m", rmf_file)
+
         if arf_file is not None and arf_file!='None':
             self.arf_file = FilePath(file_name=arf_file, file_dir=file_dir).path
+        else:
+            logger.warning("\033[31marf_file %s is not good!\033[0m", arf_file)
+
         if spec_file is not None and spec_file!='None' :
             self.spec_file = FilePath(file_name=spec_file, file_dir=file_dir).path
-
+        else:
+            logger.warning("\033[31mspec_file %s is not good!\033[0m", spec_file)
 
         self.chain_file_path = FilePath(file_name='xspec_fit.chain', file_dir=file_dir)
         self.work_dir = file_dir
@@ -709,9 +716,17 @@ class SpectralFitProduct(BaseQueryProduct):
         s = xsp.Spectrum(self.spec_file)
 
         if self.rmf_file is not None:
-            s.response = self.rmf_file.encode('utf-8')
+            s.response = self.rmf_file
+            logger.info("\033[32mrmf_file %s is good!\033[0m", self.rmf_file)
+        else:
+            logger.warning("\033[31mrmf_file %s is not good!\033[0m", self.rmf_file)
+
+
         if self.arf_file is not None:
-            s.response.arf = self.arf_file.encode('utf-8')
+            s.response.arf = self.arf_file
+            logger.info("\033[32marf_file %s is good!\033[0m", self.arf_file)
+        else:
+            logger.warning("\033[31marf_file %s is not good!\033[0m", self.arf_file)
 
         #s.ignore('**-15.')
         #s.ignore('300.-**')
