@@ -26,10 +26,9 @@ from collections import OrderedDict
 from .parameters import *
 from .products import SpectralFitProduct, QueryOutput, QueryProductList, ImageProduct
 from .io_helper import FilePath
-from .exceptions import RequestNotUnderstood
+from .exceptions import RequestNotUnderstood, UnfortunateRequestResults, BadRequest, InternalError
 import traceback
 
-
 @decorator.decorator
 def check_is_base_query(func,prod_list,*args, **kwargs):
     _check_is_base_query(prod_list)
@@ -257,18 +256,18 @@ class BaseQuery(object):
 
 class SourceQuery(BaseQuery):
     def __init__(self, name):
-        src_name = Name(name_format='str', name='src_name', value='test')
-        RA = Angle(value=0., units='deg', name='RA', )
-        DEC = Angle(value=0., units='deg', name='DEC')
+        src_name = Name(name_format='str', name='src_name', value='1E 1740.7-2942')
+        RA = Angle(value=265.97845833, units='deg', name='RA', )
+        DEC = Angle(value=-29.74516667, units='deg', name='DEC')
 
-        sky_coords = ParameterTuple([RA,DEC],'sky_coords')
+        sky_coords = ParameterTuple([RA, DEC], 'sky_coords')
 
-        t1 = Time(value='2001-12-11T00:00:00.0',name='T1',Time_format_name='T_format')
-        t2 = Time(value='2001-12-11T00:00:00.0',name='T2',Time_format_name='T_format')
+        t1 = Time(value='2017-03-06T13:26:48.000', name='T1', Time_format_name='T_format')
+        t2 = Time(value='2017-03-06T15:32:27.000', name='T2', Time_format_name='T_format')
 
         t_range = ParameterRange(t1, t2, 'time')
 
-        token = Name(name_format='str', name='token',value=None)
+        token = Name(name_format='str', name='token', value=None)
 
         #time_group = ParameterGroup([t_range_iso, t_range_mjd], 'time_range', selected='t_range_iso')
         #time_group_selector = time_group.build_selector('time_group_selector')
@@ -278,32 +277,29 @@ class SourceQuery(BaseQuery):
         super(SourceQuery, self).__init__(name, parameters_list)
 
 
-
 class InstrumentQuery(BaseQuery):
-    def __init__(self,name,
+    def __init__(self,
+                 name,
                  extra_parameters_list=[],
                  input_prod_list_name=None,
                  input_prod_value=None,
                  catalog_name=None,
                  catalog=None):
 
+        input_prod_list = InputProdList(value=input_prod_value, _format='names_list', name=input_prod_list_name, )
 
-
-        input_prod_list= InputProdList(value=input_prod_value,_format='names_list', name=input_prod_list_name, )
-
-        catalog=UserCatalog(value=catalog,name_format='str',name=catalog_name)
+        catalog=UserCatalog(value=catalog, name_format='str', name=catalog_name)
 
         selected_catalog = UserCatalog(value=None, name_format='str', name='selected_catalog')
 
         self.input_prod_list_name = input_prod_list_name
         self.catalog_name = catalog_name
 
-        parameters_list=[catalog,input_prod_list,selected_catalog]
+        parameters_list=[catalog, input_prod_list, selected_catalog]
 
-        if extra_parameters_list is not None and extra_parameters_list!=[]:
+        if extra_parameters_list is not None and extra_parameters_list != []:
             parameters_list.extend(extra_parameters_list)
 
-
         super(InstrumentQuery, self).__init__(name,parameters_list)
 
 
@@ -379,11 +375,9 @@ class ProductQuery(BaseQuery):
             query_out.set_done(message=message, debug_message=str(debug_message),status=status)
 
         except ConnectionError as e:
-            e_message = f'test of communication with backend (instrument: {instrument}) failed!'
-            e_message += "\n" + repr(e)
+            e_message = f'Connection with the backend (instrument: {instrument.name}, product: {self.name}) failed!\n' + repr(e)
 
-
-            if hasattr(e, 'debug_message'):
+            if hasattr(e, 'debug_message') and e.debug_message is not None:
                 debug_message = e.debug_message
             else:
                 debug_message = 'no exception default debug message'
@@ -392,13 +386,15 @@ class ProductQuery(BaseQuery):
             debug_message += traceback.format_exc()
 
             query_out.set_failed('dataserver communication test',
-                                 extra_message=e_message,
                                  logger=logger,
                                  sentry_client=sentry_client,
                                  excep=e,
                                  e_message=e_message,
                                  debug_message=debug_message)
 
+        except Exception as e:
+            raise InternalError(None)
+
         status = query_out.get_status()
         msg_str = '--> data server communication status: %d' %status
         logger.info(msg_str)
@@ -446,10 +442,11 @@ class ProductQuery(BaseQuery):
                                      sentry_client=sentry_client)
 
         except Exception as e:
+            # TODO same approach used above, can be used also here
             traceback.print_exc()
             print(traceback.format_exc())
             raise
-
+            # TODO all this code below con be removed
             e_message = getattr(e, 'message', 'no input products found')
             debug_message = getattr(e, 'debug_message', '')
 
@@ -500,6 +497,8 @@ class ProductQuery(BaseQuery):
 
                 if job_status=='done':
                     job.set_done()
+                elif job_status == 'failed':
+                    job.set_failed()
                 else:
                     job.set_submitted()
 
@@ -525,6 +524,7 @@ class ProductQuery(BaseQuery):
             raise
 
         except Exception as e: # TODO: could we avoid these? they make error tracking hard
+            # TODO we could use the very same approach used when test_communication fails
             logger.exception("failed to get query products")
 
             #status=1
@@ -765,9 +765,10 @@ class PostProcessProductQuery(ProductQuery):
 
         return query_out
 
+
 class ImageQuery(ProductQuery):
     def __init__(self,name,parameters_list=[],**kwargs):
-        detection_th = DetectionThreshold(value=0.0,units='sigma', name='detection_threshold')
+        detection_th = DetectionThreshold(value=7.0, units='sigma', name='detection_threshold')
         if parameters_list != [] and parameters_list is not None:
             parameters_list.append(detection_th)
         else:
@@ -778,6 +779,7 @@ class ImageQuery(ProductQuery):
         parameters_list.extend([image_scale_min, image_scale_max])
         super(ImageQuery, self).__init__(name, parameters_list, **kwargs)
 
+
 class LightCurveQuery(ProductQuery):
     def __init__(self,name,parameters_list=[], **kwargs):
 
@@ -841,6 +843,10 @@ class SpectralFitQuery(PostProcessProductQuery):
         rmf_file=instrument.get_par_by_name('rmf_file_name').value
         arf_file=instrument.get_par_by_name('arf_file_name').value
 
+        logger.info("\033[31mprocess_product: ph_file: %s\033[0m", ph_file)
+        logger.info("\033[31mprocess_product: rmf_file: %s\033[0m", rmf_file)
+        logger.info("\033[31mprocess_product: arf_file: %s\033[0m", arf_file)        
+
         e_min_kev=np.float(instrument.get_par_by_name('E1_keV').value)
         e_max_kev=np.float(instrument.get_par_by_name('E2_keV').value)
 
